@@ -10,7 +10,7 @@ import rdt
 from faker import Faker
 
 from sdv.constraints.base import Constraint
-from sdv.metadata.errors import MetadataError
+from sdv.metadata.errors import MetadataError, MetadataNotFittedError
 
 LOGGER = logging.getLogger(__name__)
 
@@ -19,16 +19,83 @@ class Table:
     """Table Metadata.
 
     The Metadata class provides a unified layer of abstraction over the metadata
-    of a single Table, which includes both the necessary details to load the data
-    from the filesystem and to know how to parse and transform it to numerical data.
+    of a single Table, which includes all the necessary details to handle the
+    table of this data, including the data types, the fields with pii information
+    and the constraints that affect this data.
+
+    Args:
+        field_names (list[str]):
+            List of names of the fields that need to be modeled
+            and included in the generated output data. Any additional
+            fields found in the data will be ignored and will not be
+            included in the generated output.
+            If ``None``, all the fields found in the data are used.
+        field_types (dict[str, dict]):
+            Dictinary specifying the data types and subtypes
+            of the fields that will be modeled. Field types and subtypes
+            combinations must be compatible with the SDV Metadata Schema.
+        field_transformers (dict[str, str]):
+            Dictinary specifying which transformers to use for each field.
+            Available transformers are:
+
+                * ``integer``: Uses a ``NumericalTransformer`` of dtype ``int``.
+                * ``float``: Uses a ``NumericalTransformer`` of dtype ``float``.
+                * ``categorical``: Uses a ``CategoricalTransformer`` without gaussian noise.
+                * ``categorical_fuzzy``: Uses a ``CategoricalTransformer`` adding gaussian noise.
+                * ``one_hot_encoding``: Uses a ``OneHotEncodingTransformer``.
+                * ``label_encoding``: Uses a ``LabelEncodingTransformer``.
+                * ``boolean``: Uses a ``BooleanTransformer``.
+                * ``datetime``: Uses a ``DatetimeTransformer``.
+
+        anonymize_fields (dict[str, str]):
+            Dict specifying which fields to anonymize and what faker
+            category they belong to.
+        primary_key (str):
+            Name of the field which is the primary key of the table.
+        constraints (list[Constraint, dict]):
+            List of Constraint objects or dicts.
+        dtype_transformers (dict):
+            Dictionary of transformer templates to be used for the
+            different data types. The keys must be any of the `dtype.kind`
+            values, `i`, `f`, `O`, `b` or `M`, and the values must be
+            either RDT Transformer classes or RDT Transformer instances.
+        model_kwargs (dict):
+            Dictionary specifiying the kwargs that need to be used in
+            each tabular model when working on this table. This dictionary
+            contains as keys the name of the TabularModel class and as
+            values a dictionary containing the keyword arguments to use.
+            This argument exists mostly to ensure that the models are
+            fitted using the same arguments when the same Table is used
+            to fit different model instances on different slices of the
+            same table.
+        name (str):
+            Name of this table. Optional.
     """
 
     _hyper_transformer = None
-    _anonymization_mappings = None
     _fakers = None
     _constraint_instances = None
+    fitted = False
 
-    _FIELD_TEMPLATES = {
+    _ANONYMIZATION_MAPPINGS = dict()
+    _TRANSFORMER_TEMPLATES = {
+        'integer': rdt.transformers.NumericalTransformer(dtype=int),
+        'float': rdt.transformers.NumericalTransformer(dtype=float),
+        'categorical': rdt.transformers.CategoricalTransformer,
+        'categorical_fuzzy': rdt.transformers.CategoricalTransformer(fuzzy=True),
+        'one_hot_encoding': rdt.transformers.OneHotEncodingTransformer,
+        'label_encoding': rdt.transformers.LabelEncodingTransformer,
+        'boolean': rdt.transformers.BooleanTransformer,
+        'datetime': rdt.transformers.DatetimeTransformer,
+    }
+    _DTYPE_TRANSFORMERS = {
+        'i': 'integer',
+        'f': 'float',
+        'O': 'one_hot_encoding',
+        'b': 'boolean',
+        'M': 'datetime',
+    }
+    _DTYPES_TO_TYPES = {
         'i': {
             'type': 'numerical',
             'subtype': 'integer',
@@ -47,7 +114,7 @@ class Table:
             'type': 'datetime',
         }
     }
-    _DTYPES = {
+    _TYPES_TO_DTYPES = {
         ('categorical', None): 'object',
         ('boolean', None): 'bool',
         ('numerical', None): 'float',
@@ -94,20 +161,37 @@ class Table:
         except AttributeError:
             raise ValueError('Category "{}" couldn\'t be found on faker'.format(category))
 
-    def __init__(self, field_names=None, field_types=None, anonymize_fields=None,
-                 primary_key=None, constraints=None, transformer_templates=None,
-                 model_kwargs=None):
+    def __init__(self, name=None, field_names=None, field_types=None, field_transformers=None,
+                 anonymize_fields=None, primary_key=None, constraints=None,
+                 dtype_transformers=None, model_kwargs=None):
+        self._name = name or str(id(self))
         self._field_names = field_names
         self._field_types = field_types or {}
+        self._field_transformers = field_transformers or {}
         self._anonymize_fields = anonymize_fields or {}
         self._model_kwargs = model_kwargs or {}
 
         self._primary_key = primary_key
         self._constraints = constraints or []
-        self._transformer_templates = transformer_templates or {}
+        self._dtype_transformers = self._DTYPE_TRANSFORMERS.copy()
+        if dtype_transformers:
+            self._dtype_transformers.update(dtype_transformers)
+
+    def __repr__(self):
+        return 'Table(name={}, field_names={})'.format(self._name, self._field_names)
 
     def get_model_kwargs(self, model_name):
-        """Return the required model kwargs for the indicated model."""
+        """Return the required model kwargs for the indicated model.
+
+        Args:
+            model_name (str):
+                Qualified Name of the model for which model kwargs
+                are needed.
+
+        Returns:
+            dict:
+                Keyword arguments to use on the indicated model.
+        """
         return copy.deepcopy(self._model_kwargs.get(model_name))
 
     def set_model_kwargs(self, model_name, model_kwargs):
@@ -117,7 +201,7 @@ class Table:
     def _get_field_dtype(self, field_name, field_metadata):
         field_type = field_metadata['type']
         field_subtype = field_metadata.get('subtype')
-        dtype = self._DTYPES.get((field_type, field_subtype))
+        dtype = self._TYPES_TO_DTYPES.get((field_type, field_subtype))
         if not dtype:
             raise MetadataError(
                 'Invalid type and subtype combination for field {}: ({}, {})'.format(
@@ -140,7 +224,7 @@ class Table:
 
         Args:
             ids (bool):
-                Whether or not include the id fields. Defaults to ``False``.
+                Whether or not to include the id fields. Defaults to ``False``.
 
         Returns:
             dict:
@@ -177,16 +261,22 @@ class Table:
 
             field_meta = self._field_types.get(field_name)
             if field_meta:
-                # Validate the given meta
-                self._get_field_dtype(field_name, field_meta)
+                dtype = self._get_field_dtype(field_name, field_meta)
             else:
                 dtype = data[field_name].dtype
-                field_template = self._FIELD_TEMPLATES.get(dtype.kind)
+                field_template = self._DTYPES_TO_TYPES.get(dtype.kind)
                 if field_template is None:
-                    raise ValueError('Unsupported dtype {} in column {}'.format(dtype, field_name))
+                    msg = 'Unsupported dtype {} in column {}'.format(dtype, field_name)
+                    raise ValueError(msg)
 
                 field_meta = copy.deepcopy(field_template)
 
+            field_transformer = self._field_transformers.get(field_name)
+            if field_transformer:
+                field_meta['transformer'] = field_transformer
+            else:
+                field_meta['transformer'] = self._dtype_transformers.get(np.dtype(dtype).kind)
+
             anonymize_category = self._anonymize_fields.get(field_name)
             if anonymize_category:
                 field_meta['pii'] = True
@@ -207,18 +297,17 @@ class Table:
             dict:
                 mapping of field names and transformer instances.
         """
-        transformer_templates = {
-            'i': rdt.transformers.NumericalTransformer(dtype=int),
-            'f': rdt.transformers.NumericalTransformer(dtype=float),
-            'O': rdt.transformers.CategoricalTransformer,
-            'b': rdt.transformers.BooleanTransformer,
-            'M': rdt.transformers.DatetimeTransformer,
-        }
-        transformer_templates.update(self._transformer_templates)
-
         transformers = dict()
         for name, dtype in dtypes.items():
-            transformer_template = transformer_templates[np.dtype(dtype).kind]
+            field_metadata = self._fields_metadata.get(name, {})
+            transformer_template = field_metadata.get('transformer')
+            if transformer_template is None:
+                transformer_template = self._dtype_transformers[np.dtype(dtype).kind]
+                field_metadata['transformer'] = transformer_template
+
+            if isinstance(transformer_template, str):
+                transformer_template = self._TRANSFORMER_TEMPLATES[transformer_template]
+
             if isinstance(transformer_template, type):
                 transformer = transformer_template()
             else:
@@ -234,8 +323,7 @@ class Table:
         for idx, constraint in enumerate(self._constraints):
             if isinstance(constraint, type):
                 constraint = constraint().to_dict()
-
-            if isinstance(constraint, Constraint):
+            elif isinstance(constraint, Constraint):
                 constraint = constraint.to_dict()
 
             constraint = Constraint.from_dict(constraint)
@@ -251,10 +339,13 @@ class Table:
         First get the ``dtypes`` and then use them to build a transformer dictionary
         to be used by the ``HyperTransformer``.
 
+        Args:
+            data (pandas.DataFrame):
+                Data to transform.
+
         Returns:
             rdt.HyperTransformer
         """
-        # dtypes = self.get_dtypes(ids=False)
         dtypes = {}
         fields = self._fields_metadata
         for column in data.columns:
@@ -318,19 +409,22 @@ class Table:
     def _make_anonymization_mappings(self, data):
         mappings = {}
         for name, field_metadata in self._fields_metadata.items():
-            if field_metadata.get('pii'):
+            if field_metadata['type'] != 'id' and field_metadata.get('pii'):
                 faker = self._get_faker(field_metadata['pii_category'])
 
                 uniques = data[name].unique()
                 fake_values = [faker() for _ in range(len(uniques))]
                 mappings[name] = dict(zip(uniques, fake_values))
 
-        self._anonymization_mappings = mappings
+        self._ANONYMIZATION_MAPPINGS[id(self)] = mappings
+
+        return mappings
 
     def _anonymize(self, data):
-        if self._anonymization_mappings:
+        anonymization_mappings = self._ANONYMIZATION_MAPPINGS.get(id(self))
+        if anonymization_mappings:
             data = data.copy()
-            for name, mapping in self._anonymization_mappings.items():
+            for name, mapping in anonymization_mappings.items():
                 data[name] = data[name].map(mapping)
 
         return data
@@ -353,6 +447,7 @@ class Table:
 
         data = self._fit_transform_constraints(data)
         self._fit_hyper_transformer(data)
+        self.fitted = True
 
     def transform(self, data):
         """Transform the given data.
@@ -365,7 +460,11 @@ class Table:
             pandas.DataFrame:
                 Transformed data.
         """
-        data = self._anonymize(data[self._field_names])
+        if not self.fitted:
+            raise MetadataNotFittedError()
+
+        fields = self.get_dtypes(ids=False)
+        data = self._anonymize(data[fields])
 
         for constraint in self._constraints:
             data = constraint.transform(data)
@@ -382,6 +481,9 @@ class Table:
         Returns:
             pandas.DataFrame
         """
+        if not self.fitted:
+            raise MetadataNotFittedError()
+
         reversed_data = self._hyper_transformer.reverse_transform(data)
 
         for constraint in self._constraints:
@@ -389,11 +491,15 @@ class Table:
 
         fields = self._fields_metadata
         for name, dtype in self.get_dtypes(ids=True).items():
-            field_type = fields[name]['type']
-            if field_type == 'id':
-                field_data = pd.Series(np.arange(len(reversed_data)))
-            else:
+            field_metadata = fields[name]
+            field_type = field_metadata['type']
+            if field_type != 'id':
                 field_data = reversed_data[name]
+            elif field_metadata.get('pii', False):
+                faker = self._get_faker(field_metadata['pii_category'])
+                field_data = pd.Series([faker() for _ in range(len(reversed_data))])
+            else:
+                field_data = pd.Series(np.arange(len(reversed_data)))
 
             reversed_data[name] = field_data.dropna().astype(dtype)
 
@@ -428,7 +534,10 @@ class Table:
         """
         return {
             'fields': copy.deepcopy(self._fields_metadata),
-            'constraints': [constraint.to_dict() for constraint in self._constraints],
+            'constraints': [
+                constraint if isinstance(constraint, dict) else constraint.to_dict()
+                for constraint in self._constraints
+            ],
             'model_kwargs': copy.deepcopy(self._model_kwargs),
         }
 
@@ -452,8 +561,10 @@ class Table:
         """
         instance = cls()
         instance._fields_metadata = copy.deepcopy(metadata_dict['fields'])
+        instance._field_names = list(instance._fields_metadata.keys())
         instance._constraints = copy.deepcopy(metadata_dict.get('constraints', []))
         instance._model_kwargs = copy.deepcopy(metadata_dict.get('model_kwargs'))
+        instance._primary_key = metadata_dict.get('primary_key')
         return instance
 
     @classmethod
