@@ -17,8 +17,8 @@ DEFAULT_MODEL_KWARGS = {
 }
 
 
-class NotFittedError(Exception):  # noqa:D101
-    pass
+class NotFittedError(Exception):
+    """Error to raise when sample is called and SDV is not fitted."""
 
 
 class SDV:
