@@ -8,6 +8,10 @@ from sdv.metadata import Table
 LOGGER = logging.getLogger(__name__)
 
 
+class NonParametricError(Exception):
+    """Exception to indicate that a model is not parametric."""
+
+
 class BaseTabularModel():
     """Base class for all the tabular models.
 
@@ -19,20 +23,32 @@ class BaseTabularModel():
             List of names of the fields that need to be modeled
             and included in the generated output data. Any additional
             fields found in the data will be ignored and will not be
-            included in the generated output, except if they have
-            been added as primary keys or fields to anonymize.
+            included in the generated output.
             If ``None``, all the fields found in the data are used.
-        primary_key (str, list[str] or dict[str, dict]):
-            Specification about which field or fields are the
-            primary key of the table and information about how
-            to generate them.
         field_types (dict[str, dict]):
             Dictinary specifying the data types and subtypes
             of the fields that will be modeled. Field types and subtypes
             combinations must be compatible with the SDV Metadata Schema.
+        field_transformers (dict[str, str]):
+            Dictinary specifying which transformers to use for each field.
+            Available transformers are:
+
+                * ``integer``: Uses a ``NumericalTransformer`` of dtype ``int``.
+                * ``float``: Uses a ``NumericalTransformer`` of dtype ``float``.
+                * ``categorical``: Uses a ``CategoricalTransformer`` without gaussian noise.
+                * ``categorical_fuzzy``: Uses a ``CategoricalTransformer`` adding gaussian noise.
+                * ``one_hot_encoding``: Uses a ``OneHotEncodingTransformer``.
+                * ``label_encoding``: Uses a ``LabelEncodingTransformer``.
+                * ``boolean``: Uses a ``BooleanTransformer``.
+                * ``datetime``: Uses a ``DatetimeTransformer``.
+
         anonymize_fields (dict[str, str]):
             Dict specifying which fields to anonymize and what faker
             category they belong to.
+        primary_key (str):
+            Name of the field which is the primary key of the table.
+        constraints (list[Constraint, dict]):
+            List of Constraint objects or dicts.
         table_metadata (dict or metadata.Table):
             Table metadata instance or dict representation.
             If given alongside any other metadata-related arguments, an
@@ -41,57 +57,33 @@ class BaseTabularModel():
             arguments or learned from the data.
     """
 
-    TRANSFORMER_TEMPLATES = None
+    _DTYPE_TRANSFORMERS = None
 
     _metadata = None
-    _field_names = None
-    _primary_key = None
-    _field_types = None
-    _anonymize_fields = None
-    _constraints = None
-
-    def __init__(self, field_names=None, primary_key=None, field_types=None,
-                 anonymize_fields=None, table_metadata=None, constraints=None):
-        if table_metadata is not None:
-            if isinstance(table_metadata, dict):
-                table_metadata = Table(table_metadata)
 
+    def __init__(self, field_names=None, field_types=None, field_transformers=None,
+                 anonymize_fields=None, primary_key=None, constraints=None, table_metadata=None):
+        if table_metadata is None:
+            self._metadata = Table(
+                field_names=field_names,
+                primary_key=primary_key,
+                field_types=field_types,
+                anonymize_fields=anonymize_fields,
+                constraints=constraints,
+                dtype_transformers=self._DTYPE_TRANSFORMERS,
+            )
+            self._metadata_fitted = False
+        else:
             for arg in (field_names, primary_key, field_types, anonymize_fields, constraints):
                 if arg:
                     raise ValueError(
                         'If table_metadata is given {} must be None'.format(arg.__name__))
 
+            if isinstance(table_metadata, dict):
+                table_metadata = Table.from_dict(table_metadata)
+
             self._metadata = table_metadata
-
-        else:
-            self._field_names = field_names
-            self._primary_key = primary_key
-            self._field_types = field_types
-            self._anonymize_fields = anonymize_fields
-            self._constraints = constraints
-
-    def _fit_metadata(self, data):
-        """Generate a new Table metadata and fit it to the data.
-
-        The information provided will be used to create the Table instance
-        and then the rest of information will be learned from the given
-        data.
-
-        Args:
-            data (pandas.DataFrame):
-                Data to learn from.
-        """
-        metadata = Table(
-            field_names=self._field_names,
-            primary_key=self._primary_key,
-            field_types=self._field_types,
-            anonymize_fields=self._anonymize_fields,
-            constraints=self._constraints,
-            transformer_templates=self.TRANSFORMER_TEMPLATES,
-        )
-        metadata.fit(data)
-
-        self._metadata = metadata
+            self._metadata_fitted = table_metadata.fitted
 
     def fit(self, data):
         """Fit this model to the data.
@@ -106,8 +98,8 @@ class BaseTabularModel():
                 the path to a CSV file which can be loaded using
                 ``pandas.read_csv``.
         """
-        if self._metadata is None:
-            self._fit_metadata(data)
+        if not self._metadata_fitted:
+            self._metadata.fit(data)
 
         self._num_rows = len(data)
 
@@ -131,7 +123,7 @@ class BaseTabularModel():
         """
         return self._metadata
 
-    def sample(self, num_rows=None, values=None, max_retries=100):
+    def sample(self, num_rows=None, max_retries=100):
         """Sample rows from this table.
 
         Args:
@@ -139,10 +131,6 @@ class BaseTabularModel():
                 Number of rows to sample. If not given the model
                 will generate as many rows as there were in the
                 data passed to the ``fit`` method.
-            values (dict):    <- FUTURE
-                Fixed values to use for knowledge-based sampling.
-                In case the model does not support knowledge-based
-                sampling, a discard+resample strategy will be used.
             max_retries (int):
                 Number of times to retry sampling discarded rows.
                 Defaults to 100.
@@ -201,26 +189,25 @@ class BaseTabularModel():
                 If the model is not parametric or cannot be described
                 using a simple dictionary.
         """
-        raise NotImplementedError()
+        raise NonParametricError()
 
-    @classmethod
-    def from_parameters(cls):
+    def set_parameters(self, parameters):
         """Regenerate a previously learned model from its parameters.
 
         Subclasses which are not parametric, such as DeepLearning
         based models, raise a NonParametricError indicating that
         this method is not supported for their implementation.
 
-        Returns:
-            BaseTabularModel:
-                New instance with its parameters set.
+        Args:
+            dict:
+                Model parameters.
 
         Raises:
             NonParametricError:
                 If the model is not parametric or cannot be described
                 using a simple dictionary.
         """
-        raise NotImplementedError()
+        raise NonParametricError()
 
     def save(self, path):
         """Save this model instance to the given path using pickle.
