@@ -1,6 +1,7 @@
 """Wrapper around CTGAN model."""
 
-import rdt
+import contextlib
+import io
 
 from sdv.tabular.base import BaseTabularModel
 
@@ -13,26 +14,32 @@ class CTGAN(BaseTabularModel):
             List of names of the fields that need to be modeled
             and included in the generated output data. Any additional
             fields found in the data will be ignored and will not be
-            included in the generated output, except if they have
-            been added as primary keys or fields to anonymize.
+            included in the generated output.
             If ``None``, all the fields found in the data are used.
-        primary_key (str, list[str] or dict[str, dict]):
-            Specification about which field or fields are the
-            primary key of the table and information about how
-            to generate them.
         field_types (dict[str, dict]):
             Dictinary specifying the data types and subtypes
             of the fields that will be modeled. Field types and subtypes
             combinations must be compatible with the SDV Metadata Schema.
-        anonymize_fields (dict[str, str or tuple]):
+        field_transformers (dict[str, str]):
+            Dictinary specifying which transformers to use for each field.
+            Available transformers are:
+
+                * ``integer``: Uses a ``NumericalTransformer`` of dtype ``int``.
+                * ``float``: Uses a ``NumericalTransformer`` of dtype ``float``.
+                * ``categorical``: Uses a ``CategoricalTransformer`` without gaussian noise.
+                * ``categorical_fuzzy``: Uses a ``CategoricalTransformer`` adding gaussian noise.
+                * ``one_hot_encoding``: Uses a ``OneHotEncodingTransformer``.
+                * ``label_encoding``: Uses a ``LabelEncodingTransformer``.
+                * ``boolean``: Uses a ``BooleanTransformer``.
+                * ``datetime``: Uses a ``DatetimeTransformer``.
+
+        anonymize_fields (dict[str, str]):
             Dict specifying which fields to anonymize and what faker
-            category they belong to. If arguments for the faker need to be
-            passed to fine tune the value generation a tuple can be passed,
-            where the first element is the category and the rest are additional
-            positional arguments for the Faker.
-        constraints (list[dict]):
-            List of dicts specifying field and inter-field constraints.
-            TODO: Format TBD
+            category they belong to.
+        primary_key (str):
+            Name of the field which is the primary key of the table.
+        constraints (list[Constraint, dict]):
+            List of Constraint objects or dicts.
         table_metadata (dict or metadata.Table):
             Table metadata instance or dict representation.
             If given alongside any other metadata-related arguments, an
@@ -56,22 +63,21 @@ class CTGAN(BaseTabularModel):
             Wheight Decay for the Adam Optimizer. Defaults to 1e-6.
         batch_size (int):
             Number of data samples to process in each step.
+        verbose (bool):
+            Whether to print fit progress on stdout. Defaults to ``False``.
     """
 
     _CTGAN_CLASS = None
     _model = None
 
-    HYPERPARAMETERS = {
-        'TBD'
-    }
-    TRANSFORMER_TEMPLATES = {
-        'O': rdt.transformers.LabelEncodingTransformer
+    _DTYPE_TRANSFORMERS = {
+        'O': 'label_encoding'
     }
 
-    def __init__(self, field_names=None, primary_key=None, field_types=None,
-                 anonymize_fields=None, constraints=None, table_metadata=None,
+    def __init__(self, field_names=None, field_types=None, field_transformers=None,
+                 anonymize_fields=None, primary_key=None, constraints=None, table_metadata=None,
                  epochs=300, log_frequency=True, embedding_dim=128, gen_dim=(256, 256),
-                 dis_dim=(256, 256), l2scale=1e-6, batch_size=500):
+                 dis_dim=(256, 256), l2scale=1e-6, batch_size=500, verbose=False):
         super().__init__(
             field_names=field_names,
             primary_key=primary_key,
@@ -87,7 +93,7 @@ class CTGAN(BaseTabularModel):
         except ImportError as ie:
             ie.msg += (
                 '\n\nIt seems like `ctgan` is not installed.\n'
-                'Please install it using:\n\n    pip install ctgan'
+                'Please install it using:\n\n    pip install sdv[ctgan]'
             )
             raise
 
@@ -98,12 +104,13 @@ class CTGAN(BaseTabularModel):
         self._batch_size = batch_size
         self._epochs = epochs
         self._log_frequency = log_frequency
+        self._verbose = verbose
 
-    def _fit(self, data):
+    def _fit(self, table_data):
         """Fit the model to the table.
 
         Args:
-            data (pandas.DataFrame):
+            table_data (pandas.DataFrame):
                 Data to be learned.
         """
         self._model = self._CTGAN_CLASS(
@@ -118,22 +125,32 @@ class CTGAN(BaseTabularModel):
             for field, meta in self._metadata.get_fields().items()
             if meta['type'] == 'categorical'
         ]
-        self._model.fit(
-            data,
-            epochs=self._epochs,
-            discrete_columns=categoricals,
-            log_frequency=self._log_frequency,
-        )
 
-    def _sample(self, size):
-        """Sample ``size`` rows from the model.
+        if self._verbose:
+            self._model.fit(
+                table_data,
+                epochs=self._epochs,
+                discrete_columns=categoricals,
+                log_frequency=self._log_frequency,
+            )
+        else:
+            with contextlib.redirect_stdout(io.StringIO()):
+                self._model.fit(
+                    table_data,
+                    epochs=self._epochs,
+                    discrete_columns=categoricals,
+                    log_frequency=self._log_frequency,
+                )
+
+    def _sample(self, num_rows):
+        """Sample the indicated number of rows from the model.
 
         Args:
-            size (int):
+            num_rows (int):
                 Amount of rows to sample.
 
         Returns:
             pandas.DataFrame:
                 Sampled data.
         """
-        return self._model.sample(size)
+        return self._model.sample(num_rows)
